
;
; FUNCTIONS
;

DirectDrawInit proc hWnd:HWND    
      
    ; Create DirectDraw COM Object
    invoke DirectDrawCreate,NULL,addr g_pDD,NULL    
    ; Check return Value
    CHECK_DD
    
    mov eax,TRUE    
    ret

DirectDrawInit endp

CreateDirectDrawSurfaces proc USES ecx bFullScreen:BOOL       
    LOCAL ddsd:DDSURFACEDESC
    
    ; Save State of Window (Windowed/FullScreen)
    push bFullScreen
    pop g_bFullScreen
    
    .if (bFullScreen == TRUE)
    ;{
        ; Set coorperative for full Screen mode
        DDINVOKE SetCooperativeLevel,g_pDD,g_hWnd, DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN or DDSCL_NOWINDOWCHANGES
        ;Check Return Value
        CHECK_DD
        
        ; Set Full Screen At Resulotion of 640x480x16
        mov gWidth,640
        mov gHeight,480        
        DDINVOKE SetDisplayMode,g_pDD,gWidth,gHeight,16
        CHECK_DD
    ;}    
    .else
    ;{
        ; Create Windowed Mode
        DDINVOKE SetCooperativeLevel,g_pDD,g_hWnd,DDSCL_NORMAL
        ;Check Return Value
        CHECK_DD        
    ;}     
    .endif
    
    invoke RtlZeroMemory,addr ddsd,SIZEOF DDSURFACEDESC
    mov ddsd.dwSize,SIZEOF DDSURFACEDESC
    
    .if(bFullScreen == TRUE)
    ;{
        ;FullScreen Mode         
        mov ddsd.dwFlags,DDSD_CAPS or DDSD_BACKBUFFERCOUNT
        mov ddsd.ddsCaps.dwCaps, DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_COMPLEX
        mov ddsd.dwBackBufferCount,1 ; 1 Back Buffer
        ;Create the PrimarySurface
        DDINVOKE CreateSurface,g_pDD,addr ddsd,addr g_pDDS,NULL
        ;Return Value
        CHECK_DD
       
        mov ddsd.ddsCaps.dwCaps, DDSCAPS_BACKBUFFER
        ; Create BackBuffer
        DDSINVOKE GetAttachedSurface,g_pDDS,addr ddsd.ddsCaps,addr g_pDDSBack
        CHECK_DD
    ;}
    .else
    ;{
        ;Windowed Mode        
        mov ddsd.dwFlags, DDSD_CAPS
        mov ddsd.ddsCaps.dwCaps, DDSCAPS_PRIMARYSURFACE
        ;Create the PrimarySurface
        DDINVOKE CreateSurface,g_pDD,ADDR ddsd,ADDR g_pDDS,NULL        
        CHECK_DD
        
        mov ddsd.dwFlags, DDSD_WIDTH or DDSD_HEIGHT or DDSD_CAPS
        mov ddsd.ddsCaps.dwCaps, DDSCAPS_OFFSCREENPLAIN
        ; 320x240 Size of backBuffer
        mov ddsd.dwWidth, 320
        mov ddsd.dwHeight, 240
        ;Create the BackBuffer
        DDINVOKE CreateSurface,g_pDD,addr ddsd,addr g_pDDSBack,NULL
        CHECK_DD ; Check Return Value
    ;}    
    .endif
    
    ;Create Clipper (Window Mode)
    .if (!bFullScreen)
        DDINVOKE CreateClipper,g_pDD,0,addr g_pClipper,NULL
        CHECK_DD
        DDCINVOKE SetHWnd,g_pClipper,0,g_hWnd
        CHECK_DD
        DDSINVOKE SetClipper,g_pDDS,g_pClipper
        CHECK_DD
    .endif
    
    ;Retrieve Surface Information
    DDSINVOKE mLock,g_pDDSBack, \
                   NULL,addr ddsd, \
                   DDLOCK_WRITEONLY or DDLOCK_WAIT or DDLOCK_SURFACEMEMORYPTR, \ 
                   NULL
    .if (eax>=0)
    ;{                     
        mov eax,ddsd.lPitch
        mov ecx,ddsd.dwHeight
        mul ecx
        invoke RtlZeroMemory,ddsd.lpSurface,eax
        DDSINVOKE UnLock,g_pDDSBack,NULL
        CHECK_DD
    ;}
    .else
    ;{
        mov eax,FALSE
        ret
    ;}
    .endif
    
    .if (g_pDDSBack!=NULL)
    ;{
        DDSINVOKE mLock,g_pDDSBack,NULL,addr ddsd,DDLOCK_WAIT, NULL
        CHECK_DD
        mov eax,ddsd.ddpfPixelFormat.dwRGBBitCount
        mov g_iBpp,eax
        DDSINVOKE UnLock,g_pDDSBack,NULL
        CHECK_DD
    ;}    
    .endif
    
    invoke RtlZeroMemory,ddsd.lpSurface,SIZEOF DDSURFACEDESC
    mov ddsd.dwSize,SIZEOF DDSURFACEDESC
    
    ; Return TRUE on successful fucntion
    mov eax,TRUE
    ret

CreateDirectDrawSurfaces endp

DestroyDirectDrawObjects proc
    
    .if (g_pDD!=NULL)

        .if (g_pDDS!=NULL)
            DDSINVOKE Release,g_pDDS
            mov g_pDDS,NULL
        .endif
        
        .if (g_pDDSBack!=NULL)
            DDSINVOKE Release,g_pDDSBack
            mov g_pDDSBack,NULL
        .endif
        
        .if(g_pClipper!=NULL)
            DDCINVOKE Release,g_pClipper
            mov g_pClipper,NULL
        .endif
        
        DDINVOKE Release,g_pDD
        mov g_pDD,NULL
    .endif
    
    ret

DestroyDirectDrawObjects endp

CheckSurfaces proc
    
    .if (g_pDDS!=NULL)
    ;{
        DDSINVOKE IsLost,g_pDDS
        .if (eax == DDERR_SURFACELOST)
           DDSINVOKE Restore,g_pDDS
        .endif
    ;}
    .endif    
    
    
    .if (g_pDDSBack!=NULL)
    ;{
        DDSINVOKE IsLost,g_pDDSBack
        .if (eax == DDERR_SURFACELOST)
           DDSINVOKE Restore,g_pDDSBack
        .endif
    ;}
    .endif 
    
    ret

CheckSurfaces endp

DDClear proc m_pDDS:LPDIRECTDRAWSURFACE,x1:DWORD,y1:DWORD,x2:DWORD,y2:DWORD
    LOCAL ddbfx:DDBLTFX
    LOCAL rcDest:RECT
    
    ; Check if NULL
    .if (!m_pDDS)
        ret
    .endif
    
    ; Clear the ddbfx Struct
    invoke RtlZeroMemory,ADDR ddbfx,SIZEOF DDBLTFX
    mov ddbfx.dwSize, SIZEOF DDBLTFX
    
    invoke CreateRGB,0,0,0 ; Black     
    mov ddbfx.dwFillColor,eax 
    invoke SetRect, addr rcDest, x1, y1, x2, y2
    ; Blit the Surface with the Primary
    DDSINVOKE Blt,m_pDDS,addr rcDest,0, 0, DDBLT_WAIT or DDBLT_COLORFILL,addr ddbfx
    
    ret

DDClear endp

DDFlip proc
    LOCAL	rcSrc:RECT   ;  // Source blit rectangle
	LOCAL	rcDest:RECT  ;  // Destination blit rectangle
	LOCAL	p:POINT   
    
    .if (!g_bFullScreen)
        mov p.x,0
        mov p.y,0
        invoke ClientToScreen,g_hWnd,addr p
        invoke GetClientRect,g_hWnd,addr rcDest
        invoke OffsetRect,addr rcDest,p.x,p.y
        invoke SetRect,addr rcSrc,0,0,320,240
        DDSINVOKE Blt,g_pDDS,addr rcDest,g_pDDSBack, addr rcSrc, DDBLT_WAIT,NULL
    .else
        DDSINVOKE Flip, g_pDDS,NULL, DDFLIP_WAIT
    .endif
    
    ret

DDFlip endp

DDPutPixel proc uses eax ecx m_pDDS:LPDIRECTDRAWSURFACE,x:DWORD,y:DWORD,r:DWORD,g:DWORD,b:DWORD
    LOCAL ddbfx:DDBLTFX
    LOCAL rcDest:RECT
    
    .if (!m_pDDS)
        ret
    .endif
    
    invoke RtlZeroMemory,ADDR ddbfx,SIZEOF DDBLTFX
    mov ddbfx.dwSize, sizeof DDBLTFX
    invoke CreateRGB,r,g,b    
    mov ddbfx.dwFillColor,eax
    mov eax,x
    add eax,1
    mov ecx,y
    add ecx,1
    invoke SetRect, addr rcDest, x, y, eax, ecx
    ; Blit the Surface with the Primary
    DDSINVOKE Blt,m_pDDS,addr rcDest,NULL, NULL, DDBLT_WAIT or DDBLT_COLORFILL,addr ddbfx    
    ret

DDPutPixel endp

CreateRGB proc uses eax ecx edx r:DWORD, g:DWORD, b:DWORD
    
    .if (g_iBpp==8) ; 8 Bit
       xor eax,eax
    .elseif (g_iBpp==16) ; 16Bit
    ;{
       mov eax,r
       shl eax,8
       mov ecx,g
       shl ecx,1
       or eax,ecx
       mov edx,b
       shr edx,3 
       or eax,ecx
    ;}      
    .elseif (g_iBpp==32) || (g_iBpp==24) ; 24/32Bit
    ;{
        mov eax,r
        shl eax,4
        mov ecx,g
        shl ecx,8
        or eax,ecx
        mov edx,b
        or eax,edx
    ;}
    .endif
    
    ret

CreateRGB endp

DrawScene proc uses eax eax esi ecx
    LOCAL y,x,posx,posy,iOffset:DWORD
    LOCAL ddsd:DDSURFACEDESC
    LOCAL szSurface:DWORD
    
    ;  Clear the ddsd struct
    invoke RtlZeroMemory,addr ddsd,SIZEOF DDSURFACEDESC
    mov ddsd.dwSize,SIZEOF DDSURFACEDESC
    
    ; Lock the Back Surface
    DDSINVOKE mLock,g_pDDSBack,NULL,addr ddsd,DDLOCK_WAIT or DDLOCK_SURFACEMEMORYPTR, NULL
    ;CHECK_DD
    mov eax,ddsd.lpSurface
    mov szSurface,eax
        
    mov y,0
Loop_y:
    mov x,0
Loop_x:  
Check_Pixel:
    mov eax,y
    imul eax,64
    add eax,x
    cmp byte ptr Screen[eax],1
    jne White_Pixel
Black_Pixel:
    xor eax,eax
    mov al,0FFh
    jmp Draw_Pixels
White_Pixel:       
    xor eax,eax
Draw_Pixels:
    mov ecx,x  
    imul ecx,STRETCHX
    mov posx,ecx
Loop_posx:
   mov ecx,y
   imul ecx,STRETCHY
   mov posy,ecx
Loop_posy:
    ; ======= Put Pixel color at surface using pointer =======
      mov ecx,posy
      imul ecx,ddsd.lPitch           
      add ecx,posx             
      mov esi,ecx
      add esi,szSurface
      mov byte ptr [esi],al ; Pixel color
    ; ========================================================
    mov ecx,y
    imul ecx,STRETCHY
    add ecx,STRETCHY
    add posy,1
    cmp posy,ecx    
    jne Loop_posy   
    
    mov ecx,x
    imul ecx,STRETCHX
    add ecx, STRETCHX
    add posx,1
    cmp posx,ecx
    jne Loop_posx 
      
    add x,1
    cmp x,64
    jne Loop_x
    
    add y,1
    cmp y,32
    jne Loop_y
    
    DDSINVOKE UnLock,g_pDDSBack,ddsd.lpSurface
    invoke CheckSurfaces
    invoke DDFlip
    ret
DrawScene endp